"""
Init for main richie openedx synchronization application
"""
from __future__ import unicode_literals

__version__ = "1.0.0"

default_app_config = 'richie_openedx_sync.apps.RichieOpenEdxSyncConfig'



# -*- coding: utf-8 -*-
""" Configuration as explained on tutorial
github.com/edx/edx-platform/tree/master/openedx/core/djangoapps/plugins"""
from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class RichieOpenEdxSyncConfig(AppConfig):
    """Richie Open Edx Synchronization Django Application"""

    name = "richie_openedx_sync"
    verbose_name = "Richie Open edX Sync"

    # This app can not be configured using the normal way of adding plugin application to Open edX
    # because this app uses the `SignalHandler.course_published` signal. Currently this signal is
    # not using the recommended way of configuring open edX signal so they could be used by 
    # external applications - `ENROLLMENT_TRACK_UPDATED = Signal(...)`
    # plugin_app = { ... }

    def ready(self):
        """
        Method to perform actions after apps registry is ended
        """
        # Load default settings configuration
        import richie_openedx_sync.settings

        # Register signals
        import richie_openedx_sync.signals



"""
Signals reveivers of Richie Open edX synchronization
"""
from django.dispatch import receiver

from xmodule.modulestore.django import SignalHandler
from student.signals import ENROLL_STATUS_CHANGE
from student.models import EnrollStatusChange


@receiver(
    SignalHandler.course_published, 
    dispatch_uid='richie_openedx_sync.signals.sync_openedx_to_richie_after_course_published')
def sync_openedx_to_richie_after_course_published(sender, course_key, **kwargs):
    """
    On updating course schedule and details on studio, update course run information on richie
    """
    from .tasks import sync_course_run_information_to_richie
    # course_key is a CourseKey object and course_id its sting representation
    sync_course_run_information_to_richie.delay(course_id=str(course_key))
    return (
        'Open edX synchronization of a course data to Richie has been triggered, '
        'from course_published signal.'
    )

@receiver(
    ENROLL_STATUS_CHANGE, 
    dispatch_uid='richie_openedx_sync.signals.sync_openedx_to_richie_after_enrollment_status_change')
def sync_openedx_to_richie_after_enrollment_status_change(
    sender,
    event=None,
    course_id=None,
    **kwargs):
    """
    On updating course enrollment changes and the change is of `enroll` type, then update course
    run information on richie
    """

    if event != EnrollStatusChange.enroll:
        return (
            'Open edX synchronization of a course data to Richie has been skipped, '
            'because the event status change is not about enroll'
        )

    from .tasks import sync_course_run_information_to_richie
    sync_course_run_information_to_richie.delay(course_id=str(course_id))
    return (
        'Open edX synchronization of a course data to Richie has been triggered, '
        'from enrollment status change signal.'
    )



"""
Script for synchronize courses to Richie marketing site
"""


from django.core.management.base import BaseCommand
from six import text_type

from xmodule.modulestore.django import modulestore
from richie_openedx_sync.tasks import sync_course_run_information_to_richie
from opaque_keys.edx.keys import CourseKey


class Command(BaseCommand):
    """
    Synchronize courses from mongo to the Richie marketing site
    """
    help = (
        'Synchronize courses from mongo to the Richie marketing site, all courses '
        'or a specific course'
    )

    def add_arguments(self, parser):
        parser.add_argument('--course_id', type=str, default=None, 
            help='Course id to synchronize, otherwise all courses would be sync')

    def handle(self, *args, **kwargs):
        """
        Execute the command
        """
        course_id = kwargs['course_id']
        courses, failed_courses = synchronize_courses(course_id=course_id)

        print("=" * 80)
        print("=" * 30 + "> Synchronization summary")
        print(u"Total number of courses to sync: {0}".format(len(courses)))
        print(u"Total number of courses which failed to sync: {0}".format(len(failed_courses)))
        print("List of sync failed courses ids:")
        print("\n".join(failed_courses))
        print("=" * 80)


def synchronize_courses(course_id=None):
    """
    Export all courses to target directory and return the list of courses which failed to export
    """
    if not course_id:
        module_store = modulestore()
        courses = module_store.get_courses()
        course_ids = [x.id for x in courses]
    else:
        course_key = CourseKey.from_string(course_id)
        course = modulestore().get_course(course_key)
        courses = [course]
        course_ids = [course_id]

    failed_courses = []

    for course_id in course_ids:
        print("-" * 80)
        print(u"Synchronizing to Richie course id = {0}".format(course_id))
        try:
            sync_course_run_information_to_richie(course_id=str(course_id))
        except Exception as err:  # pylint: disable=broad-except
            failed_courses.append(text_type(course_id))
            print(u"=" * 30 + u"> Oops, failed to sync {0}".format(course_id))
            print("Error:")
            print(err)

    return courses, failed_courses



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function

import os
import re

from setuptools import setup


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns a list of requirement strings.
    """
    requirements = set()
    for path in requirements_paths:
        requirements.update(
            line.split('#')[0].strip() for line in open(path).readlines()
            if is_requirement(line)
        )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement;
    that is, it is not blank, a comment, or editable.
    """
    # Remove whitespace at the start/end of the line
    line = line.strip()

    # Skip blank lines, comments, and editable installs
    return not (
        line == '' or
        line.startswith('-r') or
        line.startswith('#') or
        line.startswith('-e') or
        line.startswith('git+') or
        line.startswith('-c')
    )


def get_version(*file_paths):
    """Retrieves the version from the main app __init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("richie_openedx_sync", "__init__.py")

# Read the contents of your README.md file to long_description variable
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="richie-openedx-sync",
    version=version,
    author="FCT|FCCN NAU",
    author_email="info@nau.edu.pt",
    url="https://github.com/fccn/richie-openedx-sync",
    description="Richie Open edX Synchronization application",
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=load_requirements('requirements/base.in'),
    scripts=[],
    license="AGPL 3.0",
    keywords="Django, Open edX, MOOC, Richie",
    python_requires=">=3.5",
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",

        # Should match the Django minor version on requirements
        "Framework :: Django",
        "Framework :: Django :: 2.2",

        # License - should match "license" above
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",

        # Specify the Python versions - should match python_requires above
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
    ],
    platforms=["any"],
    zip_safe=False,
    packages=[
        'richie_openedx_sync',
    ],
    include_package_data=True,
    entry_points={
        "cms.djangoapp": [
            "richie_openedx_sync = richie_openedx_sync.apps:RichieOpenEdxSyncConfig",
        ],
        "lms.djangoapp": [
            "richie_openedx_sync = richie_openedx_sync.apps:RichieOpenEdxSyncConfig",
        ],
    }
)



from django.conf import settings

# Load `RICHIE_OPENEDX_SYNC_COURSE_HOOKS` setting using the open edX `ENV_TOKENS` production mode.
# This requires the `RICHIE_OPENEDX_SYNC_COURSE_HOOKS` should be added to the `EDXAPP_ENV_EXTRA`
# ansible deployment configuration.
settings.RICHIE_OPENEDX_SYNC_COURSE_HOOKS = getattr(settings, "ENV_TOKENS", {}).get(
    "RICHIE_OPENEDX_SYNC_COURSE_HOOKS",
    getattr(settings, "RICHIE_OPENEDX_SYNC_COURSE_HOOKS", None),
)



import hashlib
import hmac
import json
import requests
import logging

from celery import shared_task
from django.conf import settings
from xmodule.modulestore.django import modulestore
from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers
from opaque_keys.edx.keys import CourseKey
from student.models import CourseEnrollment


log = logging.getLogger(__name__)


@shared_task
def sync_course_run_information_to_richie(*args, **kwargs):
    """
    Synchronize an OpenEdX course run, identified by its course key, to all Richie instances.
    """
    log.info("Entering richie update course on publish")

    course_id = kwargs["course_id"]
    course_key = CourseKey.from_string(course_id)
    course = modulestore().get_course(course_key)

    if not course:
        raise ValueError("No course found with the course_id '{}'".format(course_id))

    org = course_key.org
    edxapp_domain = configuration_helpers.get_value_for_org(
        org, 
        "LMS_BASE", 
        settings.LMS_BASE
    )

    data = {
        "resource_link": "https://{:s}/courses/{!s}/info".format(
            edxapp_domain, course_key
        ),
        "start": course.start and course.start.isoformat(),
        "end": course.end and course.end.isoformat(),
        "enrollment_start": course.enrollment_start and course.enrollment_start.isoformat(),
        "enrollment_end": course.enrollment_end and course.enrollment_end.isoformat(),
        "languages": [course.language or settings.LANGUAGE_CODE],
        "enrollment_count" : CourseEnrollment.objects.filter(course_id=course_id).count()
    }

    hooks = configuration_helpers.get_value_for_org(
        org,
        'RICHIE_OPENEDX_SYNC_COURSE_HOOKS',
        getattr(settings, "RICHIE_OPENEDX_SYNC_COURSE_HOOKS", [])
    )
    if len(hooks) == 0:
        msg = (
            "No richie course hook found for organization '{}'. Please configure the "
            "'RICHIE_OPENEDX_SYNC_COURSE_HOOKS' setting or as site configuration"
        ).format(org)
        log.info(msg)
        return msg

    for hook in hooks:
        signature = hmac.new(
            hook["secret"].encode("utf-8"),
            msg=json.dumps(data).encode("utf-8"),
            digestmod=hashlib.sha256,
        ).hexdigest()

        richie_url = hook.get("url")
        timeout = int(hook.get("timeout", 5))
        
        try:
            response = requests.post(
                richie_url,
                json=data,
                headers={"Authorization": "SIG-HMAC-SHA256 {:s}".format(signature)},
                timeout=timeout,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            status_code = response.status_code
            log.debug(e, exc_info=True)
            msg = "Error synchronizing course {} to richie site {} it returned the HTTP status code {}".format(course_key, richie_url, status_code)
            log.error(msg)
            raise e
        except requests.exceptions.RequestException as e:
            log.debug(e, exc_info=True)
            msg = "Error synchronizing course {} to richie site {}".format(course_key, richie_url)
            log.error(msg)
            raise e

